#ifndef RFBDISPATCH_H
#define RFBDISPATCH_H

#include <QOpenGLContext>
#include <QCoreApplication>
#include <QImage>
#include <QThreadPool>
#include <QObject>
#include <QPoint>
#include <QSize>
#include <QColor>
#include <QMouseEvent>
#include <QVector3D>
#include <rfb/rfbclient.h>

#include "uploader.h"
#include "jpegdecoder.h"

class Dispatcher : public QObject
{
    Q_OBJECT

public:
    Dispatcher(QObject *parent = 0);

    // libvnc hooks
    static rfbBool MallocFrameBuffer(rfbClient* client);
    static void GotCopyRect(rfbClient* client, int src_x, int src_y, int w, int h, int dest_x, int dest_y);
    static void GotFillRect(rfbClient* client, int x, int y, int w, int h, uint32_t colour);
    static void GotBitmap(rfbClient* client, const uint8_t* buffer, int x, int y, int w, int h);
    static rfbBool GotJpeg(rfbClient* client, const uint8_t* buffer, int length, int x, int y, int w, int h);
    static void FinishedFrameBufferUpdate(rfbClient* client);

    void setMouse(int x, int y, bool left, bool middle, bool right);

    unsigned int decoderCount() const;
    unsigned int incrementDecoderCount();
    unsigned int decrementDecoderCount();
    int bitsPerPixel();
    Uploader &uploader();
    void setFinished(bool finished);
    bool finished() const;

    unsigned int fill_count();
    unsigned int copy_count();
    unsigned int draw_count();

    QVector3D colorMax() const;
private:
    rfbClient *m_client;
    unsigned int m_decoderCount;
    bool m_finished;
    // TODO: move this to QVnc
    Uploader m_uploader;
signals:
    void resizeFramebuffer(const QSize &size);
    void updateFramebuffer();
    void error();
public slots:
    void create_context(QOpenGLContext *context);
    void open(QString host, int port);
    void refresh();
    void terminate();
    void decodingFinished(const QImage &bitmap, const int x, const int y, const int w, const int h);
};

#endif // RFBDISPATCH_H
