#include "qvnc.h"

QVnc::QVnc()
{
    connect(this, &QVnc::windowChanged, this, &QVnc::winChanged);

    // Don't let Qt change the size of the OpenGL framebuffer
    setTextureFollowsItemSize(false);

    m_dispatcher = new Dispatcher;
    m_dispatcher->moveToThread(&m_dispatcher_thread);

    connect(&m_dispatcher_thread, &QThread::finished, m_dispatcher, &QObject::deleteLater);

    connect(this, &QVnc::open_connection, m_dispatcher, &Dispatcher::open);
    connect(this, &QVnc::terminate_connection, m_dispatcher, &Dispatcher::terminate);

    connect(m_dispatcher, &Dispatcher::updateFramebuffer, this, &QVnc::update);
    connect(m_dispatcher, &Dispatcher::error, this, &QVnc::dispatch_error);

    QThreadPool::globalInstance()->reserveThread();
    m_dispatcher_thread.start();
}

QVnc::~QVnc()
{
    m_dispatcher_thread.quit();
    m_dispatcher_thread.wait();
}

void QVnc::open()
{
    emit open_connection(this->m_host, this->m_port);
}

void QVnc::terminate()
{
    emit terminate_connection();
}

void QVnc::setHost(QString host)
{
    if (m_host == host)
        return;

    m_host = host;
    emit hostChanged(host);
}

void QVnc::setPort(int port)
{
    if (m_port == port)
        return;

    m_port = port;
    emit portChanged(port);
}

void QVnc::setMouse(int x, int y, bool left, bool middle, bool right)
{
    m_dispatcher->setMouse(x, y, left, middle, right);
}

void QVnc::dispatch_error()
{
    qDebug() << "error";
}

void QVnc::winChanged(QQuickWindow *window)
{
    if(window != NULL)
        connect(window, &QQuickWindow::openglContextCreated, m_dispatcher, &Dispatcher::create_context, Qt::BlockingQueuedConnection);
}

Dispatcher *QVnc::dispatcher() const
{
    return m_dispatcher;
}

QQuickFramebufferObject::Renderer *QVnc::createRenderer() const
{
    VncRenderer *renderer = new VncRenderer(window(),
                                            m_dispatcher->uploader().fill_buffer(),
                                            m_dispatcher->uploader().copy_textures(),
                                            m_dispatcher->uploader().copy_buffer(),
                                            m_dispatcher->uploader().draw_buffer(),
                                            m_dispatcher->uploader().draw_textures());
    connect(renderer, &VncRenderer::finishedRendering, m_dispatcher, &Dispatcher::refresh);
    connect(renderer, &VncRenderer::FBOTextureChanged, &(m_dispatcher->uploader()), &Uploader::changeFBOTexture);
    return renderer;
}

QString QVnc::host() const
{
    return m_host;
}

int QVnc::port() const
{
    return m_port;
}
