#ifndef VNC_H
#define VNC_H

#include <QQuickWindow>
#include <QObject>
#include <QQuickFramebufferObject>
#include <QThread>
#include <QOffscreenSurface>
#include <QMouseEvent>

#include "dispatcher.h"
#include "vncrenderer.h"
#include "uploader.h"

/*!
 * \brief The public interface of the VNC functionality
 */
class QVnc : public QQuickFramebufferObject
{
    Q_OBJECT
    Q_PROPERTY(QString host READ host WRITE setHost NOTIFY hostChanged)
    Q_PROPERTY(int port READ port WRITE setPort NOTIFY portChanged)

public:
    explicit QVnc();
    ~QVnc();

    Renderer *createRenderer() const;
    QString host() const;
    int port() const;
    Dispatcher *dispatcher() const;

signals:
    void open_connection(QString host, int port);
    void terminate_connection();

    void hostChanged(QString host);
    void portChanged(int port);
public slots:
    void open();
    void terminate();

    void setHost(QString host);
    void setPort(int port);
    void setMouse(int x, int y, bool left, bool middle, bool right);

    void dispatch_error();
private:
    Dispatcher *m_dispatcher;
    QThread m_dispatcher_thread;

    QString m_host;
    int m_port;
private slots:
    void winChanged(QQuickWindow *window);
};

#endif // VNC_H
