#ifndef UPLOADER_H
#define UPLOADER_H

#include <QObject>
#include <QQuickWindow>
#include <QDebug>
#include <QVector>
#include <QOffscreenSurface>
#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QOpenGLBuffer>
#include <QOpenGLTexture>
#include <QOpenGLShaderProgram>
#include <QOpenGLFramebufferObject>
#include <QThreadPool>
#include <QtAlgorithms>
#include <QSharedPointer>

class Uploader : public QObject
{
    Q_OBJECT
public:
    Uploader(QObject *parent = Q_NULLPTR);
    ~Uploader();
    void create_context(QOpenGLContext *context);
    void startedUpdate();
    void gotCopyRect(const int src_x, const int src_y, const int width, const int height, const int dest_x, const int dest_y);
    void gotFillRect(const int x, const int y, const int width, const int height, const float red, const float green, const float blue);
    void gotBitmap(const QImage &image, const int x, const int y, const int width, const int height);
    void finishedUpdate();

    void setFramebufferHeight(int framebufferHeight);

    unsigned int fill_count() const;
    unsigned int copy_count() const;
    unsigned int draw_count() const;

    QOpenGLBuffer fill_buffer();
    QSharedPointer<QVector<GLuint>> copy_textures();
    QOpenGLBuffer copy_buffer();
    QOpenGLBuffer draw_buffer();
    QSharedPointer<QVector<QOpenGLTexture *>> draw_textures();

private:
    int m_framebufferHeight;
    QOffscreenSurface m_surface;
    QOpenGLContext m_uploader_context;
    QOpenGLFunctions *m_functions;

    QOpenGLShaderProgram m_copyProgram;

    GLuint m_FBOTexture;

    unsigned int m_fill_count;
    QVector<float> m_fill_data;
    QOpenGLBuffer m_fill_buffer;

    unsigned int m_copy_count;
    QVector<float> m_copy_data;
    QSharedPointer<QVector<GLuint>> m_copy_textures;
    QOpenGLBuffer m_copy_buffer;

    unsigned int m_draw_count;
    QVector<float> m_draw_data;
    QOpenGLBuffer m_draw_buffer;
    QSharedPointer<QVector<QOpenGLTexture*>> m_draw_textures;

    uchar m_decoders_running;

public slots:
    void changeFBOTexture(const unsigned int FBOTexture, const QMatrix4x4 &ortho);
};

#endif // UPLOADER_H
