#ifndef VNCRENDERER_H
#define VNCRENDERER_H

#include <QQuickWindow>
#include <QQuickFramebufferObject>
#include <QOpenGLFramebufferObject>
#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QMatrix4x4>
#include <QOpenGLShaderProgram>
#include <QOpenGLBuffer>
#include <QOpenGLTexture>
#include <QSharedPointer>

#include "qvnc.h"

class VncRenderer : public QObject, public QQuickFramebufferObject::Renderer
{
    Q_OBJECT
public:
    VncRenderer(QQuickWindow *window,
                QOpenGLBuffer fill_buffer,
                QSharedPointer<QVector<GLuint>> copy_textures,
                QOpenGLBuffer copy_buffer,
                QOpenGLBuffer draw_buffer,
                QSharedPointer<QVector<QOpenGLTexture*>> draw_textures,
                QObject *parent = 0);
protected:
    void render() override;
    QOpenGLFramebufferObject *createFramebufferObject(const QSize &size) override;
    void synchronize(QQuickFramebufferObject *) override;
private:
    QQuickWindow *m_window;
    QOpenGLFunctions *m_functions;
    bool m_clear;

    unsigned int m_fill_count;
    QOpenGLShaderProgram m_fill_program;
    QOpenGLBuffer m_fill_buffer;

    unsigned int m_copy_count;
    QSharedPointer<QVector<GLuint>> m_copy_textures;
    QOpenGLBuffer m_copy_buffer;

    unsigned int m_draw_count;
    QOpenGLShaderProgram m_draw_program;
    QOpenGLBuffer m_draw_buffer;
    QSharedPointer<QVector<QOpenGLTexture*>> m_draw_textures;
signals:
    void FBOTextureChanged(const unsigned int FBOTexture, const QMatrix4x4 &ortho);
    void finishedRendering();
};

#endif // VNCRENDERER_H
